@@ -17,17 +17,23 @@ from typing import TYPE_CHECKING
 
 from peewee import BooleanField, CharField, DateTimeField, ForeignKeyField, TextField
 
+from darc._typing import SPHINX_BUILD
 from darc.model.abc import BaseMetaWeb as BaseMeta
 from darc.model.abc import BaseModelWeb as BaseModel
 from darc.model.utils import IntEnumField, Proxy
 from darc.model.web.hostname import HostnameModel
 
 if TYPE_CHECKING:
-    from typing import List
+    from typing import List, TypeVar
 
     from darc._compat import datetime
-    from darc.model.web.requests import RequestsModel
-    from darc.model.web.selenium import SeleniumModel
+
+    if not SPHINX_BUILD:
+        from darc.model.web.requests import RequestsModel  # pylint: disable=unused-import
+        from darc.model.web.selenium import SeleniumModel  # pylint: disable=unused-import
+    else:
+        RequestsModel = TypeVar('RequestsModel', bound='darc.model.web.requests.RequestsModel')  # type: ignore[name-defined,unreachable,misc] # pylint: disable=line-too-long
+        SeleniumModel = TypeVar('SeleniumModel', bound='darc.model.web.selenium.SeleniumModel')  # type: ignore[name-defined,unreachable,misc] # pylint: disable=line-too-long
 
 __all__ = ['URLModel', 'URLThroughModel']
 
