@@ -8,25 +8,33 @@ around managing and processing the Freenet proxy.
 
 """
 
+import contextlib
 import getpass
 import os
 import platform
-import pprint
 import shlex
-import shutil
+import signal
 import subprocess  # nosec: B404
-import sys
-import traceback
-import warnings
+import time
 from typing import TYPE_CHECKING, cast
 
-import stem.util.term as stem_term
+import psutil
 
-from darc.const import DARC_USER, DEBUG, VERBOSE
-from darc.error import FreenetBootstrapFailed, UnsupportedPlatform, render_error
+from darc.const import DARC_USER, DEBUG, getpid
+from darc.error import FreenetBootstrapFailed, UnsupportedPlatform
+from darc.logging import DEBUG as LOG_DEBUG
+from darc.logging import ERROR as LOG_ERROR
+from darc.logging import INFO as LOG_INFO
+from darc.logging import VERBOSE as LOG_VERBOSE
+from darc.logging import WARNING as LOG_WARNING
+from darc.logging import logger
 
 if TYPE_CHECKING:
-    from typing import IO
+    from io import IO  # type: ignore[attr-defined] # pylint: disable=no-name-in-module
+    from signal import Signals  # pylint: disable=no-name-in-module
+    from subprocess import Popen  # nosec: B404
+    from types import FrameType
+    from typing import NoReturn, Optional, Union
 
 # ZeroNet args
 FREENET_ARGS = shlex.split(os.getenv('FREENET_ARGS', ''))
@@ -63,13 +71,63 @@ else:
     _FREENET_ARGS = [os.path.join(FREENET_PATH, 'run.sh'), 'start']
 _FREENET_ARGS.extend(FREENET_ARGS)
 
-if DEBUG:
-    print(stem_term.format('-*- FREENET PROXY -*-', stem_term.Color.MAGENTA))  # pylint: disable=no-member
-    if _unsupported:
-        print(stem_term.format(f'unsupported system: {platform.system()}', stem_term.Color.RED))  # pylint: disable=no-member
-    else:
-        print(render_error(pprint.pformat(_FREENET_ARGS), stem_term.Color.MAGENTA))  # pylint: disable=no-member
-    print(stem_term.format('-' * shutil.get_terminal_size().columns, stem_term.Color.MAGENTA))  # pylint: disable=no-member
+if _unsupported:
+    if DEBUG:
+        logger.debug('-*- FREENET PROXY -*-')
+        logger.pline(LOG_ERROR, 'unsupported system: %s', platform.system())
+        logger.pline(LOG_DEBUG, logger.horizon)
+else:
+    logger.plog(LOG_DEBUG, '-*- FREENET PROXY -*-', object=_FREENET_ARGS)
+
+
+def launch_freenet() -> 'Popen[bytes]':
+    """Launch Freenet process.
+
+    See Also:
+        This function mocks the behaviour of :func:`stem.process.launch_tor`.
+
+    """
+    pidfile = os.path.join(FREENET_PATH, 'Freenet.pid')
+    with contextlib.suppress(OSError):
+        os.remove(pidfile)
+
+    zeronet_process = None
+    try:
+        zeronet_process = subprocess.Popen(  # pylint: disable=consider-using-with # nosec
+            _FREENET_ARGS, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+        )
+
+        def timeout_handlet(signum: 'Optional[Union[int, Signals]]' = None,
+                            frame: 'Optional[FrameType]' = None) -> 'NoReturn':
+            raise OSError('reached a %i second timeout without success' % BS_WAIT)
+
+        signal.signal(signal.SIGALRM, timeout_handlet)
+        signal.setitimer(signal.ITIMER_REAL, BS_WAIT)
+
+        while True:
+            init_line = cast(
+                'IO[bytes]', zeronet_process.stdout
+            ).readline().decode('utf-8', 'replace').strip()
+            logger.pline(LOG_VERBOSE, init_line)
+
+            if not init_line:
+                if (code := zeronet_process.returncode) is not None and code == 0:
+                    return zeronet_process
+                raise OSError(f'Process terminated: Timed out [{code}]')
+
+            if os.path.exists(pidfile):
+                pid = getpid(pidfile)
+
+                time.sleep(1)  # wait a little bit
+                if psutil.pid_exists(pid):
+                    return zeronet_process
+    except BaseException:
+        if zeronet_process is not None:
+            zeronet_process.kill()  # don't leave a lingering process
+            zeronet_process.wait()
+        raise
+    finally:
+        signal.alarm(0)  # stop alarm
 
 
 def _freenet_bootstrap() -> None:
@@ -82,6 +140,7 @@ def _freenet_bootstrap() -> None:
 
     See Also:
         * :func:`darc.proxy.freenet.freenet_bootstrap`
+        * :func:`darc.proxy.freenet.launch_freenet`
         * :data:`darc.proxy.freenet.BS_WAIT`
         * :data:`darc.proxy.freenet._FREENET_BS_FLAG`
         * :data:`darc.proxy.freenet._FREENET_PROC`
@@ -90,25 +149,7 @@ def _freenet_bootstrap() -> None:
     global _FREENET_BS_FLAG, _FREENET_PROC  # pylint: disable=global-statement
 
     # launch Freenet process
-    _FREENET_PROC = subprocess.Popen(  # nosec
-        _FREENET_ARGS, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-    )
-
-    try:
-        stdout, stderr = _FREENET_PROC.communicate(timeout=BS_WAIT)
-    except subprocess.TimeoutExpired as error:
-        stdout, stderr = error.stdout, error.stderr
-    if VERBOSE:
-        if stdout is not None:
-            print(render_error(stdout, stem_term.Color.BLUE))  # pylint: disable=no-member
-    if stderr is not None:
-        print(render_error(stderr, stem_term.Color.RED))  # pylint: disable=no-member
-
-    returncode = _FREENET_PROC.returncode
-    if returncode != 0:
-        raise subprocess.CalledProcessError(returncode, _FREENET_ARGS,
-                                            cast('IO[bytes]', _FREENET_PROC.stdout).read(),
-                                            cast('IO[bytes]', _FREENET_PROC.stderr).read())
+    _FREENET_PROC = launch_freenet()
 
     # update flag
     _FREENET_BS_FLAG = True
@@ -142,18 +183,13 @@ def freenet_bootstrap() -> None:
     if _FREENET_BS_FLAG:
         return
 
-    print(stem_term.format('-*- Freenet Bootstrap -*-',
-                                stem_term.Color.MAGENTA))  # pylint: disable=no-member
+    logger.info('-*- Freenet Bootstrap -*-')
     for _ in range(FREENET_RETRY+1):
         try:
             _freenet_bootstrap()
             break
-        except Exception as error:
+        except Exception:
             if DEBUG:
-                message = '[Error bootstraping Freenet proxy]' + os.linesep + traceback.format_exc()
-                print(render_error(message, stem_term.Color.RED), end='', file=sys.stderr)  # pylint: disable=no-member
-
-            warning = warnings.formatwarning(str(error), FreenetBootstrapFailed, __file__, 147, 'freenet_bootstrap()')
-            print(render_error(warning, stem_term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
-    print(stem_term.format('-' * shutil.get_terminal_size().columns,
-                                stem_term.Color.MAGENTA))  # pylint: disable=no-member
+                logger.ptb('[Error bootstraping Freenet proxy]')
+            logger.pexc(LOG_WARNING, category=FreenetBootstrapFailed, line='freenet_bootstrap()')
+    logger.pline(LOG_INFO, logger.horizon)
